# Replication Archive for
# 
# Daniel E. Ho, Why Affirmative Action Does Not Cause Black Students
# To Fail the Bar, 114 Yale L.J. 1997 (2005)
#
# Daniel E. Ho, Affirmative Action's Affirmative Actions: A Reply to
# Sander, 114 Yale L.J. (2005)
#
# Date: February 20, 2005
#
# Analysis file: "sander.R"
# Data file: "lsac.sav"
# Functions: "fns.R"
#
# Tested in R 1.9.0 (http://www.r-project.org/) with
# MatchIt Version 0.1-8 (http://gking.harvard.edu/matchit/)
# 
# Codebook:
#
# All of the variables take the same coding as in Richard H. Sander, A
# Systemic Analysis of Affirmative Action in American Law Schools, 57
# STAN. L. REV. 367 (2004), and as described more fully by LINDA
# F. WIGHTMAN, USER'S GUIDE: LSAC NATIONAL LONGITUDINAL DATA FILE 15
# (1999).
#
# (1) passbar = coded as 1 if student eventually passes the bar, 0
#               otherwise
# (2) zgpa = law school GPA (standardized by school)
# (3) tier = law school tier roughly in increasing order of prestige
#            from 1 to 6, where 6 represents the most competitive
#            schools
# (4) lsat = LSAT score
# (5) ugpa = undergraduate GPA (unstandardized)
# (6) male = coded as 1 if male, 0 if female
# (7) asian = coded as 1 if Asian, 0 otherwise
# (8) black = coded as 1 if black, 0 otherwise
# (9) other = coded if other minority, 0 otherwise
# (10) hisp = coded as 1 if Hispanic, 0 otherwise
#
#
#
#=====================================================================
#
# Daniel E. Ho, Why Affirmative Action Does Not Cause Black Students
# To Fail the Bar, 114 Yale L.J. 1997 (2005)
#

rm(list=ls())
library(foreign)
library(MatchIt)
library(xtable)
library(lattice)
library(car)
source("fns.R")
dta <- read.spss("lsac.sav",use.value.labels=FALSE,to.data.frame=TRUE)
names(dta) <- tolower(names(dta))
#recoding
tier2 <- factor(dta$tier)
passbar <- dta$pass_bar #or pass.bar
dta <- cbind(dta,tier2,passbar)

#Replicating the main result of Sander's Table 6.1
t61 <- glm(passbar~zgpa + lsat + tier + ugpa + male + asian + black +
           other + hisp, data=dta,family=binomial(link=logit))
#Removing post-treatment bias by excluding law school GPA
t61.post <- glm(passbar~ lsat + tier + ugpa + male + asian + black +
           other + hisp, data=dta,family=binomial(link=logit))
#displaying both results
st61.post <- as.data.frame(cbind(as.data.frame(cbind(std.coef(t61,option=4),summary(t61)$coef[-1,4])),
                                 c(NA,std.coef(t61.post,option=4)),c(NA,summary(t61.post)$coef[-1,4])))
names(st61.post) <- c("Standardized Coefficient","p-value","Standardized Coefficient","p-value")
row.names(st61.post) <- c("Law School GPA", "LSAT", "Law School Tier",
                     "Undergraduate GPA", "Male", "Asian", "Black",
                     "Other Nonwhite", "Hispanic")
xtable(st61.post,digits=c(1,2,4,2,4))

# Exact matching on all covariates
dta2 <- subset.data.frame(dta, select= c(passbar, zgpa, tier, lsat,
                                ugpa, male, asian, black, other,
                                hisp))
dta2 <- na.omit(dta2)
dta2.black <- dta2[dta2$black==1,]
exact.store <- array(NA,dim=c(5,4,2))
nn.store <- matrix(NA,5,2)
for(i in 2:6){
  #Whites
  cat("\nEffect of Tier", i, "on White Students\n")
  tdta <- dta2[dta2$tier%in%c(i,i-1),]
  twhite <- tdta$hisp!=1 & tdta$asian!=1 & tdta$other!=1 & tdta$black!=1  
  tdta <- tdta[twhite==1,]
  foo1 <- matchit(I(as.numeric(tier==i)) ~ lsat + ugpa + male, data=tdta, exact=T)
  mdta <- match.data(foo1)
  n1 <- neyman(passbar,foo1)
  glm1 <- glm(passbar ~ I(tier==i) + lsat + ugpa + male, data=mdta,
              family = binomial(link=logit))
  lsim1 <- sim.logit(glm1)
  #Blacks
  cat("\nEffect of Tier", i, "on Black Students\n")
  tdta <- dta2[dta2$tier%in%c(i,i-1) & dta2$black==1,]
  foo2 <- matchit(I(as.numeric(tier==i)) ~ lsat + ugpa + male,
                  data=tdta, exact=T)
  mdta.black <- match.data(foo2)
  n2 <- neyman(passbar,foo2)
  glm2 <- glm(passbar ~ I(as.numeric(tier==i)) + lsat + ugpa + male,
              data=mdta.black, family=binomial(link=logit))
  lsim2 <- sim.logit(glm2)
  nn.store[i-1,] <- c(nrow(mdta),nrow(mdta.black))
  exact.store[i-1,,1] <- c(lsim1$ate,lsim1$se,lsim2$ate,lsim2$se)
  exact.store[i-1,,2] <- c(n1$ate,n1$se,n2$ate,n2$se)
}
#plotting estimated causal effects
rr <- range(c(exact.store[,3,1]+2*exact.store[,4,1],
              exact.store[,3,1]-2*exact.store[,4,1],
              exact.store[,1,1]+2*exact.store[,2,1],
              exact.store[,1,1]-2*exact.store[,2,1]))
par(mar=c(2, 0, 4, 0) + 0.1, cex.lab=0.8, cex.axis=1,
    mgp=c(1,0.5,0), cex.main=1, par(mfrow=c(1,3)),
    font=6,font.main=7,font.lab=6,font.axis=6)
plot(1:5,1:5,type="n",axes=F,ylab="", xlab="",
     main = "", ylim=c(0.5,5))
title(main="Estimated causal effect on \nprobability of passing bar \nof attending:")
text(5,(5:1)-0.1,c("First tier versus second tier",
     "Second tier versus third tier",
     "Third tier versus fourth tier",
     "Fourth tier versus fifth tier",
     "Fifth tier versus HBCU"), cex=1.1, pos=2)
plot(exact.store[,1,1],1:5,ylab="", xlab="", axes=F, pch=16,
     main= "White students", cex=1,
     xlim=rr, ylim=c(0.5,5))
abline(h=1:5,col="darkgrey",lty=3)
abline(v=0,col="darkgrey")
box()
points(exact.store[,1,1],1:5,pch=16,cex=1)
segments(exact.store[,1,1]+1.96*exact.store[,2,1],1:5,
         exact.store[,1,1]-1.96*exact.store[,2,1],1:5,
         lwd=1.5)
nlab <- paste(nn.store[,1],")",sep="")
nlab[3]<-"11,177)"
for(i in 1:5){
  text(-0.2,i,as.expression(substitute(
      paste("(",italic("n")," = ",nlb,sep=""),list(nlb=nlab[i]))),cex=0.8, pos=1)
}
axis(1)
plot(exact.store[,3,1],1:5,ylab="", xlab="", axes=F, pch=16,
     main="Black students", cex=1,
     xlim=rr, ylim=c(0.5,5))
abline(h=1:5,col="darkgrey",lty=3)
abline(v=0, col="darkgrey")
points(exact.store[,3,1],1:5,pch=16,cex=1)
segments(exact.store[,3,1]+1.96*exact.store[,4,1],1:5,
         exact.store[,3,1]-1.96*exact.store[,4,1],1:5,
         lwd=1.5)
axis(1)
for(i in 1:5){
  text(-0.2,i,as.expression(substitute(
      paste("(",italic("n")," = ",nlb,")",sep=""),list(nlb=nn.store[i,2]))),cex=0.8, pos=1)
}
box()

#====================================================================
# Daniel E. Ho, Affirmative Action's Affirmative Actions: A Reply to
# Sander, 114 Yale L.J. (2005)
#

rm(list=ls())
library(foreign)
library(MatchIt)
library(xtable)
library(lattice)
library(car)
library(sem)
dta <- read.spss("lsac.sav",use.value.labels=FALSE,to.data.frame=TRUE)
names(dta) <- tolower(names(dta))

# Recoding to account for Sander's change in the dependent variable
# from eventual bar passage to first time bar passage
tier2 <- factor(dta$tier)
passbar <- dta$pass_bar #or pass.bar
# first time bar passage (passbar1)
passbar1 <- dta$bar1
passbar1 <- NA
passbar1[dta$bar1=="F"] <- 0
passbar1[dta$bar1=="P"] <- 1
passbar1[dta$bar1==" "] <- NA
# eventual bar passage (passbar2)
passbar2 <- dta$bar2
passbar2 <- NA
passbar2[dta$bar2=="F"] <- 0
passbar2[dta$bar2=="P"] <- 1
passbar2[dta$bar2==" "] <- NA
dta <- cbind(dta,tier2,passbar,passbar1,passbar2)
dta2 <- subset.data.frame(dta, select= c(passbar, passbar1, passbar2, zgpa, tier, lsat,
                                ugpa, male,black))
dta2 <- na.omit(dta2)
dta2.black <- dta2[dta2$black==1,]
rm(list=ls()[ls()!="dta2.black"])
source("fns.R")
attach(dta2.black)

# Structural Equations Approach: Reduced Forms

# (1) The effect of tier on law school GPA
lm1 <- lm(zgpa ~ lsat + tier + ugpa + male, data=dta2.black)
std.coef(lm1)
summary(lm1)

# (2) The "direct effect" of tier on first-time bar passage
lm2 <- lm(passbar1 ~ tier + ugpa + lsat + zgpa + male,
          data=dta2.black)
# Sander assumes gender does not affect first-time bar passage, but
# substantively leading to the same results 
# lm2 <- lm(passbar1 ~ tier + ugpa + lsat + zgpa,
#          data=dta2.black)
std.coef(lm2)
summary(lm2)
# "indirect effect"
std.coef(lm1)[2]*std.coef(lm2)[4]
# "total effect"
std.coef(lm2)[1]+std.coef(lm1)[2]*std.coef(lm2)[4]

# (3) The "total effect" of course is the same as OLS by correctly
# excluding the post-treatment variable (see Holland, 1988)
lm3 <- lm(passbar1 ~ tier + ugpa + lsat + male, data=dta2.black)
std.coef(lm3)
# The coefficient is not statistically significant at the 0.01
# level that Sander presents for all the other coefficients
summary(lm3)
# Testing with the exclusion restriction on gender
#lm3 <- lm(passbar1 ~ tier + ugpa + lsat, data=dta2.black)
#std.coef(lm3)

# (4) Marginal significance is driven entirely by changing the
# outcome.  Applying the same SEM to the original outcome (as
# published in the Stanford Law Review) of eventual
# bar passage
lm4 <- lm(passbar2 ~ tier + ugpa + lsat + male,
          data=dta2.black)
std.coef(lm4)
summary(lm4)
# Using more appropriate logit (since OLS assumptions are violated per
# se by the dichotomous dependent variable)
glm4 <- glm(passbar2 ~ tier + ugpa + lsat + male,
            data=dta2.black, family=binomial(link=logit))
summary(glm4)

# (5) Lastly, exact matching to assess effect on first-time bar
# passage, showing that there are no detectable effects even for
# Sander's recoded outcome without unjustified functional form
# assumptions
exact.store <- array(NA,dim=c(5,4,2))
nn.store <- matrix(NA,5,2)
for(i in 2:6){
  #Blacks
  tdta <- dta2.black[dta2.black$tier%in%c(i,i-1),]
  foo2 <- matchit(I(as.numeric(tier==i)) ~ lsat + ugpa + male,
                  data=tdta, exact=T)
  mdta.black <- match.data(foo2)
  n2 <- neyman(passbar1,foo2)
  glm2 <- glm(passbar1 ~ I(as.numeric(tier==i)) + lsat + ugpa + male,
              data=mdta.black, family=binomial(link=logit))
  lsim2 <- sim.logit(glm2)
  nn.store[i-1,] <- c(NA,nrow(mdta.black))
  exact.store[i-1,,1] <- c(NA,NA,lsim2$ate,lsim2$se)
  exact.store[i-1,,2] <- c(NA,NA,n2$ate,n2$se)
}
